@@ -4,17 +4,29 @@ use strict;
 use Carp;
 use HTML::TokeParser;
 use HTML::TokeParser::Simple::Token;
+use HTML::TokeParser::Simple::Token::Tag;
+use HTML::TokeParser::Simple::Token::Text;
+use HTML::TokeParser::Simple::Token::Comment;
+use HTML::TokeParser::Simple::Token::Declaration;
+use HTML::TokeParser::Simple::Token::ProcessInstruction;
 
 use vars qw/ @ISA $VERSION $REVISION /;
 
-$REVISION = '$Id: Simple.pm,v 1.4 2004/09/18 18:35:54 ovid Exp $';
-$VERSION  = '3.0';
+$REVISION = '$Id: Simple.pm,v 1.6 2004/09/19 21:08:17 ovid Exp $';
+$VERSION  = '3.1';
 @ISA = qw/ HTML::TokeParser /;
 
-use constant TOKEN_CLASS => 'HTML::TokeParser::Simple::Token';
-
 # constructors
 
+my %FACTORY_CLASSES = (
+    S   => 'HTML::TokeParser::Simple::Token::Tag',
+    E   => 'HTML::TokeParser::Simple::Token::Tag',
+    T   => 'HTML::TokeParser::Simple::Token::Text',
+    C   => 'HTML::TokeParser::Simple::Token::Comment',
+    D   => 'HTML::TokeParser::Simple::Token::Declaration',
+    PI  => 'HTML::TokeParser::Simple::Token::ProcessInstruction',
+);
+
 sub new {
     my ($class, @args) = @_;
     return 1 == @args
@@ -44,18 +56,24 @@ sub _init {
 
 sub get_token {
     my $self = shift;
-    my ( @args ) = @_;
+    my @args = @_;
     my $token = $self->SUPER::get_token( @args );
     return unless defined $token;
-    bless $token, TOKEN_CLASS;
+    if (my $factory_class = $FACTORY_CLASSES{$token->[0]}) {
+        return $factory_class->new($token);
+    }
+    else {
+        # this should never happen
+        croak("Cannot determine token class for token (@$token)");
+    }
 }
 
 sub get_tag {
     my $self = shift;
-    my ( @args ) = @_;
+    my @args = @_;
     my $token = $self->SUPER::get_tag( @args );
     return unless defined $token;
-    bless $token, TOKEN_CLASS;
+    return HTML::TokeParser::Simple::Token::Tag->new($token);
 }
 
 1;
@@ -64,7 +82,7 @@ __END__
 
 =head1 NAME
 
-HTML::TokeParser::Simple - easy to use HTML::TokeParser interface
+HTML::TokeParser::Simple - Easy to use C<HTML::TokeParser> interface
 
 =head1 SYNOPSIS
 
@@ -101,7 +119,9 @@ Since this is a subclass of C<HTML::TokeParser>, all C<HTML::TokeParser>
 methods are available.  To truly appreciate the power of this module, please
 read the documentation for C<HTML::TokeParser> and C<HTML::Parser>.
 
-=head1 C<new($source)>
+=head1 CONTRUCTORS
+
+=head2 C<new($source)>
 
 The constructor for C<HTML::TokeParser::Simple> can be used just like
 C<HTML::TokeParser>'s constructor:
@@ -112,7 +132,7 @@ C<HTML::TokeParser>'s constructor:
   # or
   my $parser = HTML::TokeParser::Simple->new(\$html_string);
 
-=head1 C<new($source_type, $source)>
+=head2 C<new($source_type, $source)>
 
 If you wish to be more explicit, there is a new style of
 constructor avaiable.
@@ -133,16 +153,31 @@ As a convenience, you can also attempt to fetch the HTML directly from a URL.
 This method relies on C<LWP::Simple>.  If this module is not found or the page
 cannot be fetched, the constructor will C<croak()>.
 
-=head1 C<is_> Methods
+=head1 ACCESSORS
+
+=head2 Boolean Accessors
+
+These accessors return true or false.
 
 =over 4
 
+=item * C<is_tag([$tag])>
+
+Use this to determine if you have any tag.  An optional "tag type" may be
+passed.  This will allow you to match if it's a I<particular> tag.  The
+supplied tag is case-insensitive.
+
+ if ( $token->is_tag ) { ... }
+
+Optionally, you may pass a regular expression as an argument.
+
 =item * C<is_start_tag([$tag])>
 
 Use this to determine if you have a start tag.  An optional "tag type" may be
 passed.  This will allow you to match if it's a I<particular> start tag.  The
 supplied tag is case-insensitive.
 
+ if ( $token->is_start_tag ) { ... }
  if ( $token->is_start_tag( 'font' ) ) { ... }
 
 Optionally, you may pass a regular expression as an argument.  To match all
@@ -170,16 +205,6 @@ Or:
 
 Optionally, you may pass a regular expression as an argument.
 
-=item * C<is_tag([$tag])>
-
-Use this to determine if you have any tag.  An optional "tag type" may be
-passed.  This will allow you to match if it's a I<particular> tag.  The
-supplied tag is case-insensitive.
-
- if ( $token->is_tag ) { ... }
-
-Optionally, you may pass a regular expression as an argument.
-
 =item * C<is_text()>
 
 Use this to determine if you have text.  Note that this is I<not> to be
@@ -208,21 +233,22 @@ you?)
 Process Instructions are from XML.  This is very handy if you need to parse out
 PHP and similar things with a parser.
 
+Currently, there appear to be some problems with process instructions.  You can
+override C<HTML::TokeParser::Simple::Token::ProcessInstruction> if you need to.
+
 =item * C<is_pi()>
 
 This is a shorthand for C<is_process_instruction()>.
 
 =back
 
-=head1 The C<get_> methods
+=head2 Data Accessors
 
-=head2 Note:
-
-These were originally C<return_> methods, but that name was not only unwieldy,
-but also went against reasonable conventions.  The C<get_> methods listed
-below still have C<return_> methods available for backwards compatibility
-reasons, but they merely call their C<get_> counterpart.  For example, calling
-C<return_tag()> actually calls C<get_tag()> internally.
+Some of these were originally C<return_> methods, but that name was not only
+unwieldy, but also went against reasonable conventions.  The C<get_> methods
+listed below still have C<return_> methods available for backwards
+compatibility reasons, but they merely call their C<get_> counterpart.  For
+example, calling C<return_tag()> actually calls C<get_tag()> internally.
 
 =over 4
 
@@ -236,13 +262,17 @@ If you have a start tag, this will return a hash ref with the attribute names
 as keys and the values as the values.
 
 If you pass in an attribute name, it will return the value for just that
-attribute.  Returns C<undef> if the attribute is not found.
+attribute.  
+
+Returns false if the token is not a start tag.
 
 =item * C<get_attrseq()>
 
 For a start tag, this is an array reference with the sequence of the
 attributes, if any.
 
+Returns false if the token is not a start tag.
+
 =item * C<return_text()>
 
 This method has been heavily deprecated (for a couple of years) in favor of
@@ -250,7 +280,7 @@ C<as_is>.  Programmers were getting confused over the difference between
 C<is_text>, C<return_text>, and some parser methods such as
 C<HTML::TokeParser::get_text> and friends.
 
-Using this method still succeeds, but will now carp and will likely be removed
+Using this method still succeeds, but will now carp and B<will be removed>
 in the next major release of this module.
 
 =item * C<as_is()>
@@ -263,12 +293,17 @@ For processing instructions, this will return the token found immediately after
 the opening tag.  Example:  For <?php, "php" will be the start of the returned
 string.
 
+Note that process instruction handling appears to be incomplete in
+C<HTML::TokeParser>.
+
+Returns false if the token is not a process instruction.
+
 =back
 
-=head1 Tag munging methods
+=head1 MUTATORS
 
 The C<delete_attr()> and C<set_attr()> methods allow the programmer to rewrite
-tag attributes on the fly.  It should be noted that bad HTML will be
+start tag attributes on the fly.  It should be noted that bad HTML will be
 "corrected" by this.  Specifically, the new tag will have all attributes
 lower-cased with the values properly quoted.
 
@@ -276,15 +311,22 @@ Self-closing tags (e.g. E<lt>hr /E<gt>) are also handled correctly.  Some older
 browsers require a space prior to the final slash in a self-closed tag.  If
 such a space is detected in the original HTML, it will be preserved.
 
+Calling a mutator on an token type that does not support that property is a
+no-op.  For example:
+
+ if ($token->is_comment) {
+    $token->set_attr(foo => 'bar'); # does nothing
+ }
+
 =over 4
 
 =item * C<delete_attr($name)>
 
-This method attempts to delete the attribute specified.  It will C<croak> if
-called on anything other than a start tag.  The argument is case-insensitive,
-but must otherwise be an exact match of the attribute you are attempting to
-delete.  If the attribute is not found, the method will return without changing
-the tag.
+This method attempts to delete the attribute specified.  It will silently fail
+if called on anything other than a start tag.  The argument is
+case-insensitive, but must otherwise be an exact match of the attribute you are
+attempting to delete.  If the attribute is not found, the method will return
+without changing the tag.
 
  # <body bgcolor="#FFFFFF">
  $token->delete_attr('bgcolor');
@@ -293,15 +335,14 @@ the tag.
  
 After this method is called, if successful, the C<as_is()>, C<get_attr()>
 and C<get_attrseq()> methods will all return updated results.
- 
+
 =item * C<set_attr($name,$value)>
 
 This method will set the value of an attribute.  If the attribute is not found,
-then C<get_attrseq()> will have the new attribute listed at the end.  Two
-arguments
+then C<get_attrseq()> will have the new attribute listed at the end.
 
  # <p>
- $token->set_attr('class','some_class');
+ $token->set_attr(class => 'some_class');
  print $token->as_is;
  # <p class="some_class">
 
@@ -320,7 +361,7 @@ C<get_> methods emit, the following works:
 
   $tag->set_attr($tag->get_attr);
 
-Theoretically that's a no-op and for purposes of rendering HTML, it shoudld be.
+Theoretically that's a no-op and for purposes of rendering HTML, it should be.
 However, internally this calls C<$tag-E<gt>rewrite_tag>, so see that method to
 understand how this may affect you.
 
@@ -332,9 +373,7 @@ can do this:
     valign => 'top'
   };
   $token->set_attr($attrs) 
-    if $token->is_start_tag('td') 
-        &&  
-       $token->get_attr('class') eq 'stories';
+    if $token->is_start_tag('td') &&  $token->get_attr('class') eq 'stories';
 
 =item * C<rewrite_tag()>
 
@@ -359,17 +398,19 @@ A quick cleanup of sloppy HTML is now the following:
      print $token->as_is;
  }
 
-=head1 Important note:
+=back
 
-Some people get confused and try to call parser methods on tokens and token
-methods (those described above) on methods.  To prevent this,
-C<HTML::TokeParser::Simple> versions 1.4 and above now bless all tokens into a
-new class which inherits nothing.  Please keep this in mind while using this
-module (and many thanks to PodMaster
+=head1 PARSER VERSUS TOKENS 
+
+The parser returns tokens that are blessed into appropriate classes.  Some
+people get confused and try to call parser methods on tokens and token methods
+on the parser.  To prevent this, C<HTML::TokeParser::Simple> versions 1.4 and
+above now bless all tokens into appropriate token classes.  Please keep this in
+mind while using this module (and many thanks to PodMaster
 L<http://www.perlmonks.org/index.pl?node_id=107642> for pointing out this issue
-to me.
+to me.)
 
-=head1 Examples
+=head1 EXAMPLES
 
 =head2 Finding comments
 
@@ -451,31 +492,18 @@ the form tags.  You need to change it to "http://www.bar.com/".
      close FILE;
  }
 
-=head1 COPYRIGHT
-
-Copyright (c) 2004 by Curtis "Ovid" Poe.  All rights reserved.  This program is
-free software; you may redistribute it and/or modify it under the same terms as
-Perl itself
-
-=head1 AUTHOR
-
-Curtis "Ovid" Poe L<eop_divo_sitruc@yahoo.com>
-
-Reverse the name to email the author.
-
-=head1 BUGS
+=head1 CAVEATS
 
 For compatability reasons with C<HTML::TokeParser>, methods that return
 references are violating encapsulation and altering the references directly
 B<will> alter the state of the object.  Subsequent calls to C<rewrite_tag()>
 can thus have unexpected results.  Do not alter these references directly
 unless you are following behavior described in these docs.  In the future,
-certain methods such as C<get_attr>, C<get_attrseq> and others will likely
-return a copy of the reference rather than the original reference.  This
-behavior has not yet been changed in order to maintain compatability with
-previous versions of this module.  At the present time, your author is not
-aware of anyone taking advantage of this "feature," but it's better to be safe
-than sorry.
+certain methods such as C<get_attr>, C<get_attrseq> and others may return a
+copy of the reference rather than the original reference.  This behavior has
+not yet been changed in order to maintain compatability with previous versions
+of this module.  At the present time, your author is not aware of anyone taking
+advantage of this "feature," but it's better to be safe than sorry.
 
 Use of C<$HTML::Parser::VERSION> which is less than 3.25 may result in
 incorrect behavior as older versions do not always handle XHTML correctly.  It
@@ -491,10 +519,48 @@ token.  You can suppress this behavior with the following command:
 See the C<HTML::Parser> documentation and
 http://www.perlmonks.org/index.pl?node_id=230667 for more information.
 
-Address bug reports and comments to: L<eop_divo_sitruc@yahoo.com>.  When sending bug
-reports, please provide the version of C<HTML::Parser>, C<HTML::TokeParser>,
-C<HTML::TokeParser::Simple>, the version of Perl, and the version of the
-operating system you are using.
+=head1 BUGS
+
+There are no known bugs, but that's no guarantee.
+
+Address bug reports and comments to: E<lt>eop_divo_sitruc@yahoo.comE<gt>.  When
+sending bug reports, please provide the version of C<HTML::Parser>,
+C<HTML::TokeParser>, C<HTML::TokeParser::Simple>, the version of Perl, and the
+version of the operating system you are using.
+
+Reverse the name to email the author.
+
+=head1 SUBCLASSING
+
+You may wish to change the behavior of this module.  You probably do not want
+to subclass C<HTML::TokeParser::Simple>.  Instead, you'll want to subclass one
+of the token classes.  C<HTML::TokeParser::Simple::Token> is the base class for
+all tokens.  Global behavioral changes should go there.  Otherwise, see the
+appropriate token class for the behavior you wish to alter.
+
+=head1 SEE ALSO
+
+L<HTML::TokeParser::Simple::Token>
+
+L<HTML::TokeParser::Simple::Token::Tag>
+
+L<HTML::TokeParser::Simple::Token::Text>
+
+L<HTML::TokeParser::Simple::Token::Comment>
+
+L<HTML::TokeParser::Simple::Token::Declaration>
+
+L<HTML::TokeParser::Simple::Token::ProcessInstruction>
+
+=head1 COPYRIGHT
+
+Copyright (c) 2004 by Curtis "Ovid" Poe.  All rights reserved.  This program is
+free software; you may redistribute it and/or modify it under the same terms as
+Perl itself
+
+=head1 AUTHOR
+
+Curtis "Ovid" Poe E<lt>eop_divo_sitruc@yahoo.comE<gt>
 
 Reverse the name to email the author.
 
